;;; -*- Mode: Lisp; Extended-format: Code-Page -*-
;;; Copyright (c) 1987--2001 Xanalys Inc. All rights reserved.
;;; Modified by Dmitri Ivanov for personal and public use. 
;;;
;;; Main purposes:
;;; - Provide gestures akin MS Windows and other MS products
;;; - Provide alternatives for Meta+Control+<key> combinations since those 
;;;   could be assign to switching between Windows applications.
;;; - Provide support for some Cyrillic key combinations
;;;   (     )
;;; Standard LW bindings are commented by #| and |# 

(in-package "EDITOR")

;; This file contains the key bindings with which LispWorks was sent out.  This
;; file is NOT read in each time LispWorks runs, so any modified key bindings
;; should be put in an initialisation file or saved inside a new image.  The
;; purpose of providing this file is to give the user some examples of how key
;; bindings can be specified.

;;; Interrupt gestures that override other bindings.
;(set-interrupt-keys '("Control-g" "Control-G"))
(set-interrupt-keys '("Escape"))

;;; Since we use pc-mode editor commands, we have to guarantee all patches to be loaded
(require "pc-mode")
(require "delete-selection")
(require "selection-mode")

;;;; Self Insert
#|
;; DF30June95: don't try to bind all the two-byte chars here
(loop for i from 0 below base-char-code-limit
      for c = (code-char i)
      when (or (graphic-char-p c)
               #+Lucid
               (< 159 i base-char-code-limit))
        do (bind-key "Self Insert" c))
|#


;;; Prefix Arguments
#|
(bind-key "Set Prefix Argument" #\control-\u)

(bind-key "Negative Argument" #\meta-\-)
(bind-key "Argument Digit" #\meta-0)
(bind-key "Argument Digit" #\meta-1)
(bind-key "Argument Digit" #\meta-2)
(bind-key "Argument Digit" #\meta-3)
(bind-key "Argument Digit" #\meta-4)
(bind-key "Argument Digit" #\meta-5)
(bind-key "Argument Digit" #\meta-6)
(bind-key "Argument Digit" #\meta-7)
(bind-key "Argument Digit" #\meta-8)
(bind-key "Argument Digit" #\meta-9)
|#

;;; Help
#|
;; Davef3rdJan95: was bound to simply #\control-h
;; I'm changing it so each help sub-command may have its own binding
;; DF29Nov95: that was daft -- change it back Task-3154
(bind-key "Help" '#(#\control-\h))
(bind-key "Help" :help)

;; DF30Nov95 Removed separate bindings for the help sub-commands Task-3154.

;; 30 Mar 1995, AA - fix the control-x style ones to be control-\x to
;; ensure that they are lower case
(bind-key "What Cursor Position" #(#\control-\x #\=)) ;MJS 10Jul93: added emacs-like binding
|#

;;; Executing Commands
#|
(bind-key "Extended Command" #\meta-\x)
|#


;;; Movement
#|
(bind-key "Forward Character" #\control-\f)
(bind-key "Forward Character" :right)
(bind-key "Backward Character" #\control-\b)	; DIV: To Do: Redefine
(bind-key "Backward Character" :left)
(bind-key "Beginning of Line" #\control-\a)	; DIV: Redefined as Mark Buffer
(bind-key "Beginning of Line" :home)
#-Win32 (bind-key "Beginning of Line" :begin)
(bind-key "End of Line" #\control-\e)		; Redefined as Evaluate Defun
(bind-key "End of Line" :end)
(bind-key "Next Line" #\control-\n)		; DIV: To Do: Redefine
(bind-key "Next Line" :down)
(bind-key "Previous Line" #\control-\p)		; DIV: To Do: Redefine
(bind-key "Previous Line" :up)

(bind-key "Forward Word" #\meta-\f)             ; DIV: To Do: Redefine
(bind-key "Forward Word" #("c-Right"))
(bind-key "Backward Word" #\meta-\b)		; DIV: To Do: Redefine
(bind-key "Backward Word" #("control-Left"))
(bind-key "Forward Paragraph" #\meta-])
(bind-key "Backward Paragraph" #\meta-[)
(bind-key "Forward Sentence" #\meta-\e)
(bind-key "Backward Sentence" #\meta-\a)
(bind-key "Scroll Window Down" #\control-\v)	; DIV: To Do: Redefine
(bind-key "Scroll Window Down" :next)		; PageDn
(bind-key "Scroll Window Up" #\meta-\v)		; DIV: To Do: Redefine
(bind-key "Scroll Window Up" :prior)		; PageUp
(bind-key "Beginning of Buffer" #\meta-<)
(bind-key "Beginning of Buffer" #("c-Home"))
#-Win32 (bind-key "Beginning of Buffer" #("c-Begin"))
#-Win32 (bind-key "Beginning of Window" :home)
(bind-key "Beginning of Window" #("c-Prior"))
(bind-key "End of Buffer" #\meta->)
(bind-key "End of Buffer" #("control-End"))
(bind-key "End of Window" #("control-Next"))
|#
  ; Due to original keys will be rebound by "Previous Page" and "Next Page" 
(bind-key "Beginning of Window" #\meta-control-Prior) ; Set free
(bind-key "End of Window" #\meta-control-Next)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; DIV: Excerpt from msw-key-binds.lisp but Meta- is suppressed
;;;      Requires pc-mode.fsl and patches to be loaded beforehand

;; This set of commands derived from Emacs19 pc-select.el

(editor:bind-key "Forward Character Modifying Selection"             "Shift-Right")
(editor:bind-key "Forward Character Cancelling Selection"                  "Right")

(editor:bind-key "Backward Character Modifying Selection"             "Shift-Left")
(editor:bind-key "Backward Character Cancelling Selection"                  "Left")

(editor:bind-key "Forward Word Modifying Selection"          "Control-Shift-Right")
(editor:bind-key "Forward Word Cancelling Selection"               "Control-Right")
;(editor:bind-key "Forward Word Modifying Selection"             "Meta-Shift-Right")

(editor:bind-key "Backward Word Modifying Selection"          "Control-Shift-Left")
(editor:bind-key "Backward Word Cancelling Selection"               "Control-Left")
;(editor:bind-key "Backward Word Modifying Selection"             "Meta-Shift-Left")

(editor:bind-key "Next Line Modifying Selection"                      "Shift-Down")
(editor:bind-key "Next Line Cancelling Selection"                           "Down")


(editor:bind-key "End Of Buffer Modifying Selection"           "Control-Shift-End")
;(editor:bind-key "End Of Buffer Modifying Selection"              "Meta-Shift-End")
(editor:bind-key "End Of Buffer Cancelling Selection"                "Control-End")

(editor:bind-key "Scroll Window Down Modifying Selection"             "Shift-Next")


(editor:bind-key "Previous Line Modifying Selection"                    "Shift-Up")
(editor:bind-key "Previous Line Cancelling Selection"                         "Up")

(editor:bind-key "Beginning Of Buffer Modifying Selection"    "Control-Shift-Home")
;(editor:bind-key "Beginning Of Buffer Modifying Selection"       "Meta-Shift-Home")
(editor:bind-key "Beginning Of Buffer Cancelling Selection"         "Control-Home")

(bind-key "Scroll Window Up Moving Point"                                  "Prior");4.2
;(editor:bind-key "Scroll Window Up Cancelling Selection"                  "Prior")
(bind-key "Scroll Window Up Modifying Selection"                     "Shift-Prior")
(bind-key "Scroll Window Down Moving Point"                                 "Next")
;(editor:bind-key "Scroll Window Down Cancelling Selection"                 "Next");4.2
(bind-key "Scroll Window Down Modifying Selection"                    "Shift-Next")

(editor:bind-key "Beginning of Line Cancelling Selection"                   "Home")
#-Win32 (editor:bind-key "Beginning of Line Cancelling Selection"          "Begin")
(editor:bind-key "Beginning of Line Modifying Selection"              "Shift-Home")
#-Win32 (editor:bind-key "Beginning of Line Modifying Selection"     "Shift-Begin")
(editor:bind-key "End Of Line Modifying Selection"                     "Shift-End")
(editor:bind-key "End Of Line Cancelling Selection"                          "End")
(editor:bind-key "Forward Paragraph Cancelling Selection"           "Control-Down")
(editor:bind-key "Forward Paragraph Modifying Selection"      "Control-Shift-Down")
(editor:bind-key "Backward Paragraph Cancelling Selection"            "Control-Up")
(editor:bind-key "Backward Paragraph Modifying Selection"       "Control-Shift-Up") 
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Pages
;;; Pages - DIV: Use Ctrl+Q Ctrl+L to insert page delimiter
#|
(bind-key "Previous Page" '#(#\control-\x #\[))
(bind-key "Next Page" '#(#\control-\x #\]))
(bind-key "Mark Page" '#(#\control-\x #\control-\p))
(bind-key "Count Lines Page" '#(#\control-\x #\l))
|#
(bind-key "Previous Page" #\meta-Prior)
(bind-key "Next Page" #\meta-Next)

;;; Marking
#|
(bind-key "Set Mark" #\control-\@)
(bind-key "Set Mark" #\control-space)		; DIV: Redefined to Complete Symbol a la MS Access
#-Win32 (bind-key "Pop Mark" #\meta-control-space)	; DIV: Doesn't work on Windows
(bind-key "Mark Paragraph" #\meta-\h)
(bind-key "Mark Whole Buffer" '#(#\control-\x #\h))
(bind-key "Exchange Point and Mark" '#(#\control-\x #\control-\x))
|#
(bind-key "Set Mark" #\Control-F5)
(bind-key "Pop and Goto Mark" #\shift-F5)
(bind-key "Mark Whole Buffer" #\control-\a)


;;; Registers
#|
(bind-key "Point to Register" '#(#\control-\x #\/))
(bind-key "Jump to Register" '#(#\control-\x #\j))
(bind-key "Copy to Register" '#(#\control-\x #\x))
(bind-key "Insert Register" '#(#\control-\x #\g))
|#

;;; Case
#|
(bind-key "Uppercase Word" '#\meta-\u)
(bind-key "Lowercase Word" '#\meta-\l)
(bind-key "Capitalize Word" '#\meta-\c)
(bind-key "Uppercase Region" '#(#\control-\x #\control-\u))
(bind-key "Lowercase Region" '#(#\control-\x #\control-\l))
|#
(bind-key "Uppercase Region" #\meta-\U)
(bind-key "Lowercase Region" #\meta-\L)
(bind-key "Capitalize Region" #\meta-\C)


;;; Deletion
#|
(bind-key "Delete Next Character" #\control-\d)
(bind-key "Delete Next Character" #\Delete)
(bind-key "Delete Previous Character" #\Backspace)
(bind-key "Kill Next Word" #\meta-\d)
(bind-key "Kill Next Word" #\meta-Delete)
(bind-key "Kill Previous Word" #\meta-Backspace)
(bind-key "Kill Line" #\control-\k)
(bind-key "Extract List" #\control-\K)
(bind-key "Delete Blank Lines" '#(#\control-\x #\control-\o) :global)
(bind-key "Kill Region" #\control-\w)
(bind-key "Save Region" #\meta-\w)
(bind-key "Save Region" "Control-Insert")
(bind-key "Forward Kill Sentence" #\meta-\k)
(bind-key "Forward Kill Sentence" '#(#\control-\x #\delete))
(bind-key "Backward Kill Sentence" '#(#\control-\x #\backspace))
(bind-key "Delete Horizontal Space" #\meta-\\)
(bind-key "Just One Space" #\meta-space)
|#
;;; DIV: msw-key-binds.lisp ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(editor:bind-key "Kill Next Word" "Control-Delete")
(editor:bind-key "Kill Previous Word" "Control-Backspace")
(editor:bind-key "Save Region Preserving Selection" "Control-Insert")
(editor:bind-key "Kill Region" "Shift-Delete")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(bind-key "Kill Line" #\meta-Delete)             ; DIV: Bound to Forward Kill Form
(bind-key "Kill Line" #\control-\-)

(bind-key "Do Nothing" #\meta-\d)                 ; DIV: Dangerous

(defcommand "Delete Character Or Selection" (p)
     "Delete (not kill!) the region if it is highlighted otherwise - next character" ""
  (cond ((editor:variable-value-if-bound "Highlight Active Region" :buffer (current-buffer))
         (delete-region-command p)                ; Delete without pushing into the kill ring
         (editor::set-highlight-buffer-region nil)
         ;(editor:"Delete Selection Mode")   
         )
        ((delete-next-character-command p))))

;; Press Shift+Delete to kill the highlighted region from now on
(bind-key "Delete Character Or Selection" #\Delete)

;;; Insertion
#|
(bind-key "Overwrite Mode" :insert)

(bind-key "Un-Kill" #\control-\y)
(bind-key "Un-Kill" "Shift-Insert")
(bind-key "Rotate Kill Ring" #\meta-\y)
(bind-key "Open Line" #\Control-\o)
(bind-key "New Line" #\Return)
(bind-key "Quoted Insert" #\control-\q)
|#
(bind-key "Rotate Kill Ring" #\meta-Insert)


;;; Undo
#|
(bind-key "Undo" #\control-_)		; i.e. ctrl-shift-hyphen
(bind-key "Undo" '#(#\control-\x #\u))	
|#
(bind-key "Undo" #\control-\z)
(bind-key "Undo" #\meta-Backspace)


;;; Transposition
#|
(bind-key "Transpose Characters" #\control-\t)
(bind-key "Transpose Words" #\meta-\t)
(bind-key "Transpose Lines" '#(#\control-\x #\control-\t))
|#
(bind-key "Transpose Characters" #\control-\T)
(bind-key "Transpose Words" #\control-\t)


;;; Indentation
#|
(bind-key "Indent" #\tab)
(bind-key "Indent" #\control-\i)
(bind-key "Indent Region" #\meta-control-\\)
(bind-key "Indent Rigidly" '#(#\control-\x #\tab))
(bind-key "Indent Rigidly" '#(#\control-\x #\control-\i))
(bind-key "Delete Indentation" #\meta-\^)
(bind-key "Indent New Line" #\Linefeed)
(bind-key "Indent New Line" #\control-\j)
(bind-key "Back to Indentation" #\meta-\m)
(bind-key "Back to Indentation" #\meta-control-\m)
|#
(bind-key "Indent Region" #\control-\I)
(bind-key "Indent New Line" #\control-Return)
(bind-key "Back to Indentation" #\meta-\<)


;;; Filling
#|
(bind-key "Fill Paragraph" #\meta-\q)
(bind-key "Fill Region" #\meta-\g)
(bind-key "Set Fill Prefix" '#(#\control-\x #\.))
(bind-key "Set Fill Column" '#(#\control-\x #\f))
|#
(bind-key "Fill Region" #\meta-\Q)
(bind-key "Auto Fill Mode" #(#\control-\x #\a))


;;; Searching
#|
(bind-key "Incremental Search" #\control-\s)
(bind-key "Reverse Incremental Search" #\control-\r)

;; 24 Sep 1993, AA - bind these to their EMACS equivalents
(bind-key "Forward Search" #\meta-\s)
(bind-key "Reverse Search" #\meta-\r)

(bind-key "Query Replace" #\meta-%)

(bind-key "Next Grep" #(#\Control-\x #\`))
|#
(bind-key "Query Replace" #\control-%)   ; due to Query Replace RegExp redifinition
(bind-key "Search All Buffers" "Ctrl-F3")
(bind-key "System Search" "Meta-F3")
(bind-key "System Query Replace" "Meta-Shift-F3")
(bind-key "System Query Replace" "Meta-Shift-F3")
(bind-key "Directory Search" "Meta-Control-F3")
(bind-key "Directory Query Replace" "Meta-Control-Shift-F3")
(bind-key "Grep" #\Meta-\`)
(bind-key "Next Grep" #\Control-\`)

;;; Regular Expression Searching
#|
(bind-key "ISearch Forward Regexp" #\meta-control-\s)
(bind-key "ISearch Backward Regexp" #\meta-control-\r)
|#
(bind-key "ISearch Forward Regexp" #\meta-\s)
(bind-key "ISearch Backward Regexp" #\meta-\r)
(bind-key "Query Replace RegExp" #\meta-%)


;;; Files
#|
(bind-key "Wfind File" '#(#\control-\x #\control-\f))
(bind-key "Find Alternate File" '#(#\control-\x #\control-\v))
(bind-key "Write File" '#(#\control-\x #\control-\w))
(bind-key "Save File" '#(#\control-\x #\control-\s))
(bind-key "Save All Files" '#(#\control-\x #\s))
|#
;;; (char-code #\) -> 1099  Russian letter under the keyboard key 's'
(bind-key "Save File" '#(#\control-\x #\control-\))
(bind-key "Save All Files" '#(#\control-\x #\))
#|
(bind-key "Save All Files and Exit" '#(#\control-\x #\control-\c))
(bind-key "Insert File" '#(#\control-\x #\i))
|#
;(bind-key "Wfind File" #\control-\o)		; I would be better File>Open!
(bind-key "Load File" '(#\control-\x #\control-\l))


;;; Buffers
#|
(bind-key "Select Buffer" '#(#\control-\x #\b))
(bind-key "List Buffers" '#(#\control-\x #\control-\b))
(bind-key "Select Previous Buffer" #\meta-control-\l)
(bind-key "Kill Buffer" '#(#\control-\x #\k))
(bind-key "Buffer Not Modified" #\meta-~)
(bind-key "Check Buffer Modified" '#(#\control-\x #\~))
;; 6 May 1994 (Jason) -- changed binding from C-M-l to C-M-S-l. The
;; key-binding forms in this file actually end up being stuffed onto a
;; list and processed in reverse order. Hence the Circulate Buffers
;; binding of C-M-l was overwritten by the one for Select Previous Buffer.
(bind-key "Circulate Buffers" #\meta-control-\L)
(bind-key "Toggle Buffer Read-Only" '#(#\control-\x #\control-\q))
|#
(bind-key "Select Previous Buffer" #\F6)
(bind-key "List Buffers" #\meta-\F6)
(bind-key "Kill Buffer" #\control-\F4)           ; DIV: Always close window in MDI
(bind-key "Circulate Buffers" #\Shift-F6)
(bind-key "Toggle Buffer Read-Only" '#(#\control-\x #\r))
(bind-key "New Buffer" '#(#\control-\x #\control-\n))
(bind-key "Revert Buffer" '#(#\control-\x #\control-\r))
(bind-key "List Buffer Definitions" '#(#\control-\x #\control-\d))

;;; Windows 
#|
(bind-key "New Window" '#(#\control-\x #\2)) 
(bind-key "Next Ordinary Window" '#(#\control-\x #\o))
(bind-key "Delete Window" '#(#\control-\x #\0)) ; added 13/2/89 (MSM)
(bind-key "Delete Other Windows" '#(#\control-\x #\1))
;;(bind-key "Line to Center of Window" #\meta-\#); removed 5/90,jfw
(bind-key "Refresh Screen" #\control-\l)
;; 24 Sep 1993, AA - add this for EMACS compatibility
(bind-key "Move To Window Line" #\meta-\r)	; Redefined to Reverse Reg.Expr search
|#
;(editor:bind-key "Refresh Screen" "F5")

(bind-key "Next Window" #\control-\F6)               ; auto for MDI
(bind-key "Previous Window" #\control-shift-\F6)     ; auto for MDI
;(bind-key "Delete Window" #\meta-\F4)


;;; Abbrevs
#|
(bind-key "Add Mode Word Abbrev" '#(#\control-\x #\control-\a))
(bind-key "Add Global Word Abbrev" '#(#\control-\x #\+))
(bind-key "Inverse Add Mode Word Abbrev" '#(#\control-\x #\control-\h))
(bind-key "Inverse Add Global Word Abbrev" '#(#\control-\x #\-))
(bind-key "Word Abbrev Prefix Point" #\meta-\')
|#


;;; Recursive Editing
#|
(bind-key "Exit Recursive Edit" #\meta-control-\z)
(bind-key "Abort Recursive Edit" #\control-])
|#
(bind-key "Exit Recursive Edit" #\control-\5)

;;; Lisp Operations
#|
(bind-key "Forward Form" #\meta-control-\f)
(bind-key "Backward Form" #\meta-control-\b)
(bind-key "Forward Kill Form" #\meta-control-\k)
(bind-key "Forward Kill Form" #\meta-control-Delete) ; DIV: Don't work on Windows
(bind-key "Backward Kill Form" #\meta-control-Backspace)
(bind-key "End of Defun" #\meta-control-\e)
(bind-key "Beginning of Defun" #\meta-control-\a)
(bind-key "Indent Form" #\meta-control-\q)
(bind-key "Forward List" #\meta-control-\n)
(bind-key "Backward List" #\meta-control-\p)
(bind-key "Mark Defun" #\meta-control-\h)
(bind-key "Transpose Forms" #\meta-control-\t)
(bind-key "Insert ()" #\meta-\()
(bind-key "Move Over )" #\meta-\))
(bind-key "Backward Up List" #\meta-control-\u)
(bind-key "Down List" #\meta-control-\d)
|#
;;; Reserve Meta-Left and Meta-Right for History menu
(bind-key "Backward Form" "Meta-Control-Left"  :global :emacs)
(bind-key "Forward Form"  "Meta-Control-Right" :global :emacs)
(bind-key "Forward Kill Form" #\meta-Delete)
(bind-key "Indent Form" #\meta-\i)
(bind-key "Beginning of Defun" #\meta-Home)
(bind-key "End of Defun" #\meta-End)
(bind-key "Backward List" #\meta-[)
(bind-key "Forward List" #\meta-])
(bind-key "Backward Up List" #\meta-Up)
;(bind-key "Forward Up List" #\meta-Down)
;(bind-key "Down List" #\Clear)                 ; Shift+Num5 works even when Numlock is on
(bind-key "Transpose Forms" #\meta-\t)

(editor:bind-key "Down List" "Meta-Down")



;;; Comments
#|
(bind-key "Indent for Comment" #\meta-\;)
(bind-key "Set Comment Column" '#(#\control-\x #\;))
(bind-key "Kill Comment" #\meta-control-\;)
(bind-key "Down Comment Line" #\meta-\n)
(bind-key "Up Comment Line" #\meta-\p)
(bind-key "Indent New Comment Line" #\meta-\j)
(bind-key "Indent New Comment Line" #\meta-Linefeed)
|#


;;; Evaluation & Lisp stuff
#|
(bind-key "Evaluate Defun" '#\meta-control-\x)
(bind-key "Evaluate Expression" #\meta-Escape)
(bind-key "Evaluate Last Form" #(#\control-\x #\control-\e))

(bind-key "Evaluate Region" #\control-\E)
(bind-key "Macroexpand Form" #\control-\M)
(bind-key "Walk Form" #\meta-\M)
(bind-key "Mark Form" #\meta-control-@)

(bind-key "Compile Defun" #\control-\C) ; KPA 9/11/92
(bind-key "Compile Region" #\control-\R) ;; 20 May 1993, AA
(bind-key "Compile Buffer" #\control-\B) ;; 20 May 1993, AA
|#
(bind-key "Evaluate Defun" '#\control-\e)
(bind-key "Evaluate Defun" '#\F9)
(bind-key "Evaluate Buffer" "Control-F9")
(bind-key "Evaluate Region" "Meta-Shift-F9")
(bind-key "Evaluate Last Form" #\meta-\E)
(bind-key "Evaluate Last Form" "Shift-F9")

(bind-key "Undefine" "Meta-Control-F9")

(bind-key "Re-Evaluate Defvar" #\control-\v)
(bind-key "Evaluate Expression" #(#\control-\x #\control-\e))

(bind-key "Compile Defun" #\F12)
(bind-key "Compile Buffer File" "Control-F12")
(bind-key "Compile Region" "Meta-Shift-F12")
(bind-key "Compile and Load Buffer File" "Meta-Control-F12")

;;;; Keyboard Macros
#|
(bind-key "Define Keyboard Macro" '#(#\control-\x #\())
(bind-key "End Keyboard Macro" '#(#\control-\x #\)))
(bind-key "Last Keyboard Macro" '#(#\control-\x #\e))
(bind-key "Keyboard Macro Query" '#(#\control-\x #\q))
|#

;;; Completion
#|
(bind-key "Complete Symbol" #\meta-control-\i)
(bind-key "Dynamic Completion" #\meta-\/)
(bind-key "Expand File Name"  #\meta-Tab )         ; DIV: doesn't work on Windows
|#
(bind-key "Complete Symbol" #\control-\Space)      ; DIV: Borrowed from MS Access
(bind-key "Expand File Name" #\control-\/)        


;;;; Dspec
#|
(bind-key "Find Source" #\meta-\.)
;(bind-key "Visit Other Tags File" #\meta-\control-\.) ; 4.2
(bind-key "Rotate Active Finders" #\meta-\control-\.) ; 4.3
|#
(bind-key "Find Source" #\Shift-F2)        ; DIV: a la MS Access


;;;; Tags
#|
(bind-key "Find Tag" #\meta-\?)
(bind-key "Continue Tags Search" #\meta-\,)
|#

;;; On line documentation.
#|
(bind-key "Function Documentation" #\control-D)
(bind-key "Show Documentation" #\meta-control-\A)
(bind-key "Function Arglist" #\meta-=)
(bind-key "Function Argument List" #\control-\A)
|#
;;; DIV: a la msw-bind-keys ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(editor:bind-key "Function Documentation" "Meta-F1")   ; web manual page
(editor:bind-key "Show Documentation" "Ctrl-F1")       ; CL:DOCUMENTATION in editor help window
(editor:bind-key "Function Argument List" "F1")        ; arglist in editor echo area

(editor:bind-key "Describe Class" "F2") ; With prefix - auto use default value under point
(editor:bind-key "Describe Generic Function" "Ctrl-F2")
(editor:bind-key "Describe Symbol" "Meta-F2")

;; Online doc on Editor: F2
;(editor:bind-key "Document Command" "Ctrl-F2")
;(editor:bind-Key "Document Key" "Shift-F2")   
;(editor:bind-key "Document Variable" "Ctrl-Shift-F2")
;(bind-key "Generic Describe" #\Meta-Shift-F2)		; for some editor entity

;(editor:bind-key "Help"                                                       "F1")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;; Mail bindings
#-Harlequin-PC-Lisp(bind-key "Mail" #(#\control-\x #\m))

;;;; Echo area

;;; Basic echo-area commands
#|
(bind-key "Help on Parse" :f1 :mode "Echo Area")
(bind-key "Help on Parse" :help :mode "Echo Area")
;; DF09Jan96 don't want this -- see Task-757 and Task-3154 for explanation.
;; The ? character is picked up by a LOGICAL-CHAR= match in COMMAND-CASE, instead.
;; DF02Feb96: Restore this binding, but make command more sensitive to the type 
;; of the current-parse-inf -- in particular during prompt-for-symbol just call 
;; Self Insert. That should fix Task-757 (again!), whilst retaining emacs
;; compatibility in the general case of typing ? in the echo area.
(bind-key "Help on Parse" #\? :mode "Echo Area")
(bind-key "Complete Input" #\Tab :mode "Echo Area")
(bind-key "Complete Field" #\Space :mode "Echo Area")
(bind-key "Confirm Parse" #\Return :mode "Echo Area")
(bind-key "Kill Parse" #(#\c-\c #\c-\u) :mode "Echo Area")
(bind-key "Return Default" #(#\c-\c #\c-\r) :mode "Echo Area")
(bind-key "Insert Parse Default" #(#\c-\c #\c-\p) :mode "Echo Area")
|#
;;; Rebind some standard commands to behave better.
#|
(bind-key "Previous Parse" #\meta-\p :mode "Echo Area")
(bind-key "Next Parse" #\meta-\n :mode "Echo Area")
(bind-key "Find Matching Parse" #\meta-\r :mode "Echo Area")
(bind-key "Beginning of Parse or Line" #\control-\a :mode "Echo Area")
(bind-key "Beginning of Parse" #\meta-< :mode "Echo Area")
(bind-key "Echo Area Kill Previous Word" #\meta-Backspace :mode "Echo Area")
(bind-key "Echo Area Backward Character" #\control-\b :mode "Echo Area")
(bind-key "Echo Area Backward Word" #\meta-\b :mode "Echo Area")
(bind-key "Echo Area Delete Previous Character" #\Backspace :mode "Echo Area")
(bind-key "Reset Echo Area" #\meta-k :mode "Echo Area")
|#
(bind-key "Beginning of Parse or Line" "Home" :mode "Echo Area") ; Use Ctrl+Home to go to start
;(bind-key "Echo Area Kill Previous Word" #\control-Backspace :mode "Echo Area")
(bind-key "Undo" #\meta-Backspace :mode "Echo Area")   ; Restore usual meaning

#|;; Following are from msw-bind-keys.lisp. The 1st and 2nd are undefined commands ??
(bind-key "Echo Area Backward Character Modifying Selection" "Shift-Left" :mode "Echo Area")
(bind-key "Echo Area Backward Word Modifying Selection" "Control-Shift-Left" :mode "Echo Area")
(bind-key "Echo Area Backward Word Cancelling Selection" "Control-Left" :mode "Echo Area")
 |#

;;; From selection-key-binds.lisp
;;; Rebind some standard commands to behave better.
(bind-key "Previous Parse" "Ctrl-Up" :mode "Echo Area")
(bind-key "Next Parse" "Ctrl-Down" :mode "Echo Area")
(bind-key "Echo Area Backward Character Cancelling Selection" "Left" :mode "Echo Area")

;;; Nuke some dangerous standard bindings.
#|
(bind-key "Illegal" #\control-meta-\l :mode "Echo Area")           ; Select Previous Buffer
(bind-key "Illegal" #\meta-\x :mode "Echo Area")

;;; Y 12May95  now these work
; (bind-key "Illegal" #\control-\s :mode "Echo Area")
; (bind-key "Illegal" #\control-\r :mode "Echo Area")

(bind-key "Illegal" #(#\control-\x #\b) :mode "Echo Area")          ; Select Buffer
(bind-key "Illegal" #(#\control-\x #\control-\w) :mode "Echo Area") ; Write File
;; DF24Jan96 trying to insert file into the echo area has to be a mistake
(bind-key "Illegal" #(#\control-\x #\i) :mode "Echo Area")
;; DF24Jan96 Task-1140
(bind-key "Illegal" #(#\control-\x #\control-\s) :mode "Echo Area") ; Save File
;; DF24Jan96 unlike Ctrl-x Ctrl-f, this one would trash the entire window!
(bind-key "Illegal" #(#\control-\x #\control-\v) :mode "Echo Area") ; Visit File
;; DF02Feb96 Task-3272
(bind-key "Illegal" #(#\control-\x #\k) :mode "Echo Area")          ; Kill Buffer
|#
(bind-key "Illegal" #(#\control-\x #\control-\l) :mode "Echo Area") ; Load File
  
;;;; Miscellaneous

#-Win32(bind-key "Shell Command" #\meta-\!)
#-Win32(bind-key "Shell Command On Region" #\meta-\|) ;MJS 20/02/92


(bind-key "Select Buffer Other Window" 
           #(#\control-\x #\4 #\b))

;;; Cyrillic patch to be executed after loading Overwrite Mode
#|
(loop for i from #x0400 upto #x04ff ; Cyrillic range in Unicode
      do 
      (editor:bind-key "Self Overwrite" (code-char i) :mode "Overwrite"))
|#

;; Pekka 19Mar96: make sure Null isn't bound to a real command,
;; because function keys are all hacked to generate Null on LCL
#-Harlequin-Common-Lisp (bind-key "Illegal" #\Null)
;; DF25Mar96 This needed too. There are no mode bindings other 
;; than in Echo Area which use the chars Lucid lacks
#-Harlequin-Common-Lisp (bind-key "Illegal" #\Null :mode "Echo Area")


;;; Listener bindings moved here frm rubber.lisp DF27May99
;;;; Listener Key bindings

#|
(defmode "Execute" :major-p nil
  :owner 'execute-or-insert-newline-or-yank-from-previous-prompt-command) ; gleep


;;; Debugger

(bind-key "Debugger Backtrace" #\meta-\B :mode "Execute")
(bind-key "Debugger Abort" #\meta-\A :mode "Execute")
(bind-key "Debugger Continue" #\meta-\C :mode "Execute")
(bind-key "Debugger Previous" #\meta-\P :mode "Execute")
(bind-key "Debugger Next" #\meta-\N :mode "Execute")
(bind-key "Debugger Edit" #\meta-\E :mode "Execute")
(bind-key "Debugger Print" #\meta-\V :mode "Execute")


;;; Listener

(bind-key "Execute or Insert Newline or Yank from Previous Prompt" #\Return
          :mode "Execute")

(bind-key "Beginning Of Line After Prompt" #\control-\a :mode "Execute")
(bind-key "Throw To Top Level" #\meta-\k :mode "Execute")
(bind-key "Inspect Star" #(#\Control-\c #\Control-\i) :mode "Execute")
(bind-key "Insert From Previous Prompt" #\control-\j :mode "Execute")


;;; Listener History 

(bind-key "History Previous" #(#\control-\c #\control-\p) :mode "Execute")
(bind-key "History Next" #(#\control-\c #\control-\n) :mode "Execute")
(bind-key "History Yank" #(#\control-\c #\control-\y) :mode "Execute")
(bind-key "History Select" #(#\control-\c #\control-\f) :mode "Execute")
(bind-key "History Last" #(#\control-\c #\>) :mode "Execute")
(bind-key "History First" #(#\control-\c #\<) :mode "Execute")
(bind-key "History Search" #(#\control-\c #\control-\r) :mode "Execute")
(bind-key "History Kill Current" #(#\control-\c #\control-\k) :mode "Execute")

(bind-key "History Previous" #(#\meta-\p) :mode "Execute")
(bind-key "History Next" #(#\meta-\n) :mode "Execute")
(bind-key "History Search" #(#\meta-\r) :mode "Execute")
|#

;;; DIV: msw-key-binds.lisp ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; Listener

(editor:bind-key "Beginning Of Line After Prompt Cancelling Selection" "Home" :mode "Execute")
(editor:bind-key "Beginning Of Line After Prompt Modifying Selection" "Shift-Home" :mode "Execute")

;;; Listener History 

(editor:bind-key "History Previous" "Ctrl-Up" :mode "Execute")
(editor:bind-key "History Next" "Ctrl-Down" :mode "Execute")
(editor:bind-key "History Search" "Ctrl-Shift-Up" :mode "Execute")


;; Because we add accelerators for the following commands in PC mode, these
;; editor bindings are not actually reachable. However, they are defined here
;; so that the editor's online help (Help > Command to Key etc) remains
;; consistent 

;; "File > New"
(editor:bind-key "New Buffer" "ctrl-n")


;; "File > Open..."
(editor:bind-key "Wfind File"  "Control-o") ; not strictly correct as menu item calls
                                            ;  editor:find-file-command , with arguments. But the
				            ;  the effect is most like Wfind File, ensuring
				            ;  it reuses the current window 

;; "File > Save"
;(editor:bind-key "Save File" "Control-s")   ; not strictly correct as "File > Save" can call
                                            ;  editor:save-file-as-command

;; "File > Print..."
(editor:bind-key "Print File" "Control-p")

;; "Edit > Undo"
(editor:bind-key "Undo" #\Control-\z)

;; "Edit > Cut"
;(editor:bind-key "Kill Region" "Control-x") ; not strictly correct as menu item
                                            ;  puts text on the LispWorks clipboard, not
				            ;  the editor kill ring, but the effect is the same.

;; "Edit > Copy"
(editor:bind-key "Save Region Preserving Selection" "Control-c")
(editor:bind-key "Save Region Preserving Selection" "Control-c" :mode "Execute") 

;; "Edit > Paste"
;(editor:bind-key "Un-Kill" "Control-v")

;; "Edit > Find..."
;(editor:bind-key "Forward Search" "Control-f")
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
