;;; -*- Mode: Lisp; Extended-format: Code-Page -*-

(in-package :editor)

;;; All commands start highlighting and move the point in corresponding manner

(defun highlight-if-none ()
  (unless (variable-value-if-bound "Highlight Active Region" :buffer (current-buffer))
    ;; No highlighting yet
    (set-current-mark (current-point))
    (editor::set-highlight-buffer-region t)))        ; makes variable be bound if wasn't

(defcommand "Previous Page Highlight" (p) "" ""
  (highlight-if-none) (previous-page-command p))
(defcommand "Next Page Highlight" (p) "" ""
  (highlight-if-none) (next-page-command p))

(bind-key "Previous Page Highlight" #\meta-shift-Prior)	; does not work for Alt+Left Shift
(bind-key "Next Page Highlight" #\meta-shift-Next)	; - Windows keyboard layout switch!
	
;;;
;;; Lisp stuff

(defcommand "Backward Form Highlight" (p) "" ""
  (highlight-if-none) (backward-form-command p))
(defcommand "Forward Form Highlight" (p) "" ""
  (highlight-if-none) (forward-form-command p))

(defcommand "Beginning of Defun Highlight" (p) "" ""
  (highlight-if-none) (beginning-of-defun-command p))
(defcommand "End of Defun Highlight" (p) "" ""
  (highlight-if-none) (end-of-defun-command p))

(defcommand "Backward List Highlight" (p) "" ""
  (highlight-if-none) (backward-list-command p))
(defcommand "Forward List Highlight" (p) "" ""
  (highlight-if-none) (forward-list-command p))

(defcommand "Backward Up List Highlight" (p) "" ""
  (highlight-if-none) (backward-up-list-command p))
(defcommand "Forward Up List Highlight" (p) "" ""
  (highlight-if-none) (forward-up-list-command p))
 
(bind-key "Backward Form Highlight" "Meta-Shift-Left"	       :global :emacs)
(bind-key "Backward Form Highlight" "Meta-Control-Shift-Left"  :global :emacs)
(bind-key "Forward Form Highlight"  "Meta-Shift-Right"         :global :emacs)
(bind-key "Forward Form Highlight"  "Meta-Control-Shift-Right" :global :emacs)
(bind-key "Beginning of Defun Highlight" "Meta-Shift-Home" :global :emacs)
(bind-key "End of Defun Highlight"	 "Meta-Shift-End"  :global :emacs)
(bind-key "Backward List Highlight" #\meta-{)
(bind-key "Forward List Highlight"  #\meta-})
(bind-key "Backward Up List Highlight" #\meta-shift-Up)
(bind-key "Forward Up List Highlight" #\meta-shift-Down)

