;;; -*- Mode: Lisp; rcs-header: "$Header: /hope/cam1/hope.0/compound/23/LISPexamples/RCS/dde:lispworks-ide.lisp,v 1.4.1.1 2003/02/26 21:52:50 davef Exp $" -*-

;;; ========================================================================
;;; examples/dde/lispworks-ide.lisp
;;;
;;; This example demonstrates DDE support for Windows Explorer interaction
;;; with the LispWorks IDE.
;;;
;;; To try it, compile and load this file and then execute:
;;;
;;;      (START-LISPWORKS-IDE-SERVER)
;;;
;;; Then configure the Windows Explorer to invoke DDE as follows:
;;;
;;;   1. Choose View->Options... from any explorer window.
;;;   2. Select the File Types tab and click the New Type button.
;;;   3. Enter Lisp Source File as the description and lisp as the
;;;      associated extension.
;;;   4. Click New... in the Actions section.
;;;   5. Enter open as the action and lispworks-4100.exe -edit "%1" as the
;;;      application used to perform the action (browse for
;;;      lispworks-4100.exe if needed).
;;;   6. Click on Use DDE and enter [open("%1")] as the message, LispWorks as
;;;      the application and Editor as the topic.
;;;   7. Click OK on the New Action dialog.
;;;   8. Click Close on the Add New File Type dialog and the Options dialog.
;;;   9. Now locate a .lisp file in the explorer and double click on it.  It
;;;      should be opened in the LispWorks editor.
;;;
;;; ========================================================================
;;; Copyright (c) 1987--2003 Xanalys LLC. All rights reserved.
;;; ========================================================================

(in-package "CL-USER")

(eval-when (:compile-toplevel :load-toplevel :execute)
  (require "dde"))

(win32:define-dde-server lispworks-ide-server ()
  ()
  (:service "LispWorks"))

(win32:define-dde-dispatch-topic editor :server lispworks-ide-server)

(win32:define-dde-server-function (open :topic editor)
    :execute ((filename string))
  (when-let (path (probe-file filename))
    (ed path)))

;;; The latest version (Lisp Support Call #31693), thanks to Martin Simmons

(defun run-lispworks-ide-server-loop ()
  (win32:start-dde-server 'lispworks-ide-server)
  ;(mp:wait-processing-events most-positive-fixnum))
  (loop (mp:wait-processing-events 1)))

(push '("DDE IDE Server" () run-lispworks-ide-server-loop)
      mp:*initial-processes*)

(lw:define-action "Delivery Actions" "Remove DDE IDE Server"
  (lambda ()
    (lw:removef mp:*initial-processes* "DDE IDE Server" :key 'first :test 'equal)
    (remhash 'editor WIN32::*DDE-DISPATCH-TOPICS*)
    (clos:remove-direct-subclass (find-class 'WIN32::DDE-SERVER)
                                 (find-class 'lispworks-ide-server))
    (setf (find-class 'lispworks-ide-server) nil)
    (fmakunbound 'run-lispworks-ide-server-loop)))
    

#||
(WIN32::DDE-SERVER-DISPATCH-TOPICS 'run-lispworks-ide-server-loop)
(WIN32::DDE-SERVERS)
(find-class 'lispworks-ide-server)

(mp:find-process-from-name "DDE IDE Server")

;;; Old manner start up
(defun start-lispworks-ide-server (&optional screen)
  (when-let (lw-tools::podium (capi:locate-interface 'lw-tools::podium
                                   :screen nil :no-busy-interface t))
    (capi:execute-with-interface lw-tools::podium
                                 'win32:start-dde-server 'lispworks-ide-server)))

(define-action "Initialize LispWorks Tools" "Start DDE IDE Server"
   'start-lispworks-ide-server)
;(start-lispworks-ide-server)
||#
