;;; -*- Mode: Lisp; Extended-format: Code-Page -*-
;;; Commands usually called from menu bar.
;;; Hacks providing accelerators for them and menu commands built in LispWorks IDE.

(in-package :editor)

;; We always want these functions in IDE, though Ystok-Widgets could be a better place
(export '(capi::ambient-tab-layout capi::find-menu-item)
        :capi)

(let ((lw:*handle-warn-on-redefinition* :quiet))
 (defun capi::find-menu-item (parent path)
  ;;; path ::= (:works-menu 1 :tools-menu 0 (:MENU "Works" "Tools" "System Browser"))
  (labels ((get-items (parent)
             (cond ((CAPI::MENU-COMPONENT-P parent) (capi:collection-items parent))
                   ((CAPI::MENU-OBJECT-P parent) (capi:menu-items parent))
                   (t (capi:interface-menu-bar-items parent))))
           (get-item (parent index)
             (cond ((consp index)
                    (find index (get-items parent) :test 'equal
                                                   :key 'CAPI::MNEMONIC-SIGNATURE))
                   ((numberp index)
                    (if (CAPI::MENU-COMPONENT-P parent)
                        (capi:get-collection-item parent index)
                        (elt (get-items parent) index)))
                   (t
                    (find index (get-items parent) :key 'capi:capi-object-name)))))
    (dolist (index (if (listp path) path (list path)) parent)	; =(listify path)
      (unless (setq parent (get-item parent index))
        (return)))))

 ;;; Provide keyboard accelerators for vital tools accessible from standard menus.

 (defmethod capi:interface-display :before ((self capi-toolkit:object-interface))
  (when-let (item (capi::find-menu-item self '(:works-menu 0 (:menu "Works" "Refresh"))))
    (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\F5)))

 (defmethod capi:interface-display :before ((self capi-toolkit:history-interface))
  ;;; History menu > Previous and Next
  (when-let (item (capi::find-menu-item self '((:menu "History") 1 0)))
    (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\Meta-Left))
  (when-let (item (capi::find-menu-item self '((:menu "History") 1 1)))
    (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\Meta-Right)) )

 ;;; Assigning #\Rubout would be more convenient but it also be disabled
 ;;; in the Function and Filter panes!
 (defmethod capi:interface-display :before ((self lispworks-tools:class-browser))
  #1=(when-let (item (capi::find-menu-item self
                      '((:menu "Methods") 1 (:MENU "Methods" "Undefine..."))))
       (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\Meta-Control-F9)))

 (defmethod capi:interface-display :before
				((self lispworks-tools:generic-function-browser))
  #1#)

 ;;; Unfortunately, submenus are not created at the capi:interface-display time,
 ;;; but only after the menu is pulled-down the very first time.
 #+unused
 (defmethod capi:interface-display :before ((self lispworks-tools::lispworks-interface))
  (when-let (component (capi::find-menu-item self '(:works-menu 1 :tools-menu 0)))
    (when-let (item (capi::find-menu-item component '((:menu "Works" "Tools" "Editor"))))
      (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\F4))
    (when-let (item (capi::find-menu-item component
                                          '((:menu "Works" "Tools" "Listener"))))
      (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\Meta-Control-\l))
    (when-let (item (capi::find-menu-item component
                                          '((:menu "Works" "Tools" "System Browser"))))
      (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\F11))
 ))
 #+unused	; Does not work - is Ctrl+F4 reserved?
 (defmethod capi:interface-display :before ((self lispworks-tools:editor))
  (when-let (item (capi::find-menu-item self '(:file-menu 0 0 (:menu "File" "Close"))))
    (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\Control-F4)))

  #|(when-let (menu (find-if (lambda (menu)
                             (equal (slot-value menu 'CAPI::MNEMONIC-SIGNATURE)
                                    '(:menu "History")))
                           (capi:interface-menu-bar-items self)))
    (let ((component (second (capi:menu-items menu))))
      (setf (CAPI::MENU-ITEM-ACCELERATOR (capi:get-collection-item component 0))
            #\Meta-Left
            (CAPI::MENU-ITEM-ACCELERATOR (capi:get-collection-item component 1))
            #\Meta-Right)))|#
)

(defcommand "Tools Browse System" (p)
     "Acts like menu Works > Tools > System Browser"
  (declare (ignore p))
  (lispworks-tools::make-tools-menu-callback 'lispworks-tools:system-browser))
    
(bind-key "Tools Browse System" #\F11)
(bind-key "Describe System" "Control-F11")

(defcommand "Tools Editor" (p)
     "Acts like menu Works > Tools > Editor"
  (declare (ignore p))
  (lispworks-tools::make-tools-menu-callback 'lispworks-tools:editor))
    
(bind-key "Tools Editor" #\meta-control-\e)

(defcommand "Tools Listener" (p)
     "Acts like menu Works > Tools > Listener"
  (declare (ignore p))
  (capi:find-interface 'lw-tools:listener))
  ;(lispworks-tools::make-tools-menu-callback 'lispworks-tools:listener))
    
(bind-key "Tools Listener" #\meta-control-\l)

;;; Tab layout page switcher

(let ((lw:*handle-warn-on-redefinition* :quiet))
 (defgeneric capi::ambient-tab-layout (object)
  (:documentation
   "Return an instance of capi:tab-layout - (grand)parent for a pane or top-level for an interface.")
  (:method (object) nil)
  ;(:method ((object capi:screen)) nil)
  (:method ((object capi:tab-layout)) object)
  (:method ((object CAPI::BASIC-ELEMENT))			; recurse
   (capi::ambient-tab-layout (capi:element-parent object)))
))

(defun capi::%change-tab-page (object step &optional interface)
 ;;;  Change the visible tab page of the outer tab-layout of the capi object given if any.
  ;; Args: n ::= -1 (:prior), +1 (:next), or integer offset number in pages
  ;; NB: (capi:tab-layout-panes) seems to be rotated after each change of visible child.
  (or (lw:when-let* ((capi:tab-layout (capi::ambient-tab-layout (or object interface)))
                     (position (capi:choice-selection capi:tab-layout)))
        (when (< -1 (incf position step) (capi:count-collection-items capi:tab-layout))
          (setf (capi:choice-selection capi:tab-layout) position)
          (when (capi:callbacks-selection-callback capi:tab-layout)
            (CAPI::CAPI-COLLECTION-CALLBACK capi:tab-layout position
                 :index position
                 :callback-accessor 'capi:callbacks-selection-callback))
          position))
      (capi:beep-pane)))


(let ((lw:*handle-warn-on-redefinition* :quiet))
 (defmethod capi::ambient-tab-layout ((self capi-toolkit:view-interface))
   (capi::ambient-tab-layout (CAPI-TOOLKIT::VIEW-INTERFACE-CHOICE self)))

 (defmethod CAPI::ENSURE-MENU-BAR-ITEMS :before ((self capi-toolkit:view-interface))
  ;;; Provide keyboard accelerators for tab page by putting them into the last menu,
   ;; usually Help (find better place?)
  (when-let (menu (first (last (capi:interface-menu-bar-items self))))
    (lw:push-end (make-instance 'capi:menu-component :items
                  `(,(make-instance 'capi:menu-item :title "Prior tab" :help-key nil
                                    :data -1  :accelerator #\Control-Prior)
                    ,(make-instance 'capi:menu-item :title "Next tab"  :help-key nil
                                    :data 1  :accelerator #\Control-Next))
                  :callback 'capi::%change-tab-page
                  :callback-type '(:focus :data :interface))
                 (capi:menu-items menu))))
)

; (defun prior-tab-page (pane &optional step)
;   (%change-tab-page pane (- (or step 1))))
; (defun next-tab-page (pane &optional step)
;   (%change-tab-page pane (or step 1)))

(defcommand "Prior Tab Page" (step) "" ""
  (capi::%change-tab-page (editor::current-editor-pane) (- (or step 1))))
(defcommand "Next Tab Page" (step) "" ""
  (capi::%change-tab-page (editor::current-editor-pane) (or step 1)))

;;; Find and Replace

(defcommand "Edit Find" (p) "Acts like menu Edit > Find..." ""
  (declare (ignore p))
  (when-let (item (capi::find-menu-item capi::*top-level-interface* '(:edit-menu 3 0)))
    (funcall (capi:callbacks-selection-callback item) capi::*top-level-interface*)))
#|(let*((menu (find :edit-menu (capi:interface-menu-bar-items capi::*top-level-interface*)
                     :key #'capi:capi-object-name)))
    (when menu                                       ; menu bar is visible
      (let* ((component (elt (capi:menu-items menu) 3))
             (item (capi:get-collection-item component 0)))
        (funcall (capi:callbacks-selection-callback item)
                 capi::*top-level-interface*)))))|#
    
(defcommand "Edit Find Next" (p) "Acts like menu Edit > Find Next" ""
  (declare (ignore p))
  (when-let (item (capi::find-menu-item capi::*top-level-interface* '(:edit-menu 3 1)))
    (funcall (capi:callbacks-selection-callback item) capi::*top-level-interface*)))
#|(let*((menu (find :edit-menu (capi:interface-menu-bar-items capi::*top-level-interface*)
                     :key #'capi:capi-object-name)))
    (when menu                                       ; menu bar is visible
      (let* ((component (elt (capi:menu-items menu) 3))
             (item (capi:get-collection-item component 1))) 
        (funcall (capi:callbacks-selection-callback item)
                 capi::*top-level-interface*)))))|#
    
(defcommand "Edit Replace" (p)
     "Acts like menu Edit > Replace..." ""
  (declare (ignore p))
  (when-let (item (capi::find-menu-item capi::*top-level-interface* '(:edit-menu 3 2)))
    (funcall (capi:callbacks-selection-callback item) capi::*top-level-interface*)))
#|(let*((menu (find :edit-menu (capi:interface-menu-bar-items capi::*top-level-interface*)
                     :key #'capi:capi-object-name)))
    (when menu                                       ; menu bar is visible
      (let* ((component (elt (capi:menu-items menu) 3))
             (item (capi:get-collection-item component 2))) 
        (funcall (capi:callbacks-selection-callback item)
                 capi::*top-level-interface*)))))|#

(bind-key "Edit Find" #\control-\f)
(bind-key "Edit Find Next" "F3")
(bind-key "Edit Replace" "Ctrl-Shift-F3")


;;; Save/Restore Meta-comma search state by Nick Levin.
;;; I can be in the middle of a long search through The Code - say using
;;; M-x System Search - and then I get sidetracked by some interesting
;;; function call I find along the way and so I use M-. to locate its
;;; source. Hot damn, I've gone and lost the remains of the System Search:
;;; M-, is looking for more copies of my interesting function.
;;;
;;; Annoyed by this too many times. Hence the following implementation of
;;; M-# (flips back to previous M-, sequence).

(defstruct (meta-comma-state (:constructor make-meta-comma-state (action point)))
  action
  point)

(defvar *previous-meta-comma-state* nil)

(defmacro with-previous-meta-comma-state ((previous) &body body)
  ;; Cache current action and its point, retrieve previously cached counterparts
  `(when-let (,previous (shiftf *previous-meta-comma-state*
                                (when-let (current *meta-comma-action*)
                                  (make-meta-comma-state current
                                                         *search-files-last-point*))))
    ,@body))

(defun setup-meta-comma-action (&optional function)
  ;; Redefinition of existing LW function (tags.lisp), commented thus:
  ;; Arrange for meta-comma to call this function.
  ;; This previous definition deleted the current point,
  ;; which made recycling it later impossible.
  (with-previous-meta-comma-state (previous)
    (delete-point (meta-comma-state-point previous)))
  (setf *meta-comma-action* function))

(defun rotate-meta-comma-action ()
  (with-previous-meta-comma-state (previous)
    (setf *meta-comma-action* (meta-comma-state-action previous)
          *search-files-last-point* (meta-comma-state-point previous))
    (message "Meta-comma action flipped")))

(defcommand "Rotate Meta Comma Action" (p)
     "Flip back to previous meta-comma action" ""
  (declare (ignore p))
  (rotate-meta-comma-action))

(bind-key "Rotate Meta Comma Action" "Meta-\#")


;;; Insert User name initials and Date invoked by Meta+Shift+D

(defvar *month-names* '("Jan" "Feb" "Mar" "Apr"
                        "May" "Jun" "Jul" "Aug"
                        "Sep" "Oct" "Nov" "Dec"))
(defparameter *user-inits* "DI")

(defcommand "Insert User Stamp" (p)
     "Insert user name abbreviation and today's date at the editor current point"
     ""
  (declare (ignore p))
  (multiple-value-bind (seconds minutes hours day month year) (get-decoded-time)
    (declare (ignore seconds minutes hours))
    (editor:insert-string (editor:current-point)
                          (format nil "~A ~4D-~A-~2,'0D"
                                  *user-inits* year (nth (1- month) *month-names*) day))))

(bind-key "Insert User Stamp" #\Meta-\d)

(defcommand "Switch to or from Typeout" (p) 
     "Make a collector pane the visible child of a switchable layout or switch back" "" 
  (declare (ignore p))
  (let ((current-pane (current-editor-pane)))
    (if (typep current-pane 'capi:collector-pane) 
        (capi:unmap-typeout current-pane)
        (capi:map-typeout current-pane)))) ;(window-text-pane (current-window))))

(bind-key "Switch to/from Typeout" #\Shift-F4)

(bind-key "Inspect Star" #\Meta-\*)

(defcommand "Expression Inspect Value" (p)
     "Acts like menu Expression > Inspect Value"
  (declare (ignore p))
  (let* ((menu (find :works-expression-menu (capi:interface-menu-bar-items
                                               capi::*top-level-interface*)
                     :key #'capi:capi-object-name)))
    (when menu                                       ; menu bar is visible
      (let* ((component (elt (capi:menu-items menu) 0))
             (item (capi:get-collection-item component 5))) 
        (when (capi:menu-object-enabled item)
          (funcall (capi:callbacks-selection-callback item)
                   capi::*top-level-interface*))))))
    
(bind-key "Expression Inspect Value" #\Meta-Return)  ; a la Properties menu item


;;; DEBUGGING

;; see example for cl:step in lwref
#+lispworks4 (setq hcl:*print-step-level* t)	; in LW5 is already T

(bind-key "Trace Definition" #\F7)
(bind-key "Trace Definition Inside Definition" #\Meta-F7)
(bind-key "Untrace Definition" #\Ctrl-F7)

(bind-key "Trace Function" #\Shift-F7)
(bind-key "Trace Function Inside Definition" #\Meta-Shift-F7)
(bind-key "Untrace Function" #\Ctrl-Shift-F7)

(bind-key "Toggle Breakpoint" #\Meta-F9)

;;; Assignemnts takes effect only when the input focus is in the Source
;;; or Listener (only if mode "Execute") panes of the Stepper tool.
;(bind-key "Stepper Step"		"F8"	:mode "Stepper")    ; for function = Over
;(bind-key "Stepper Step Through Call" "Meta-F8"	:mode "Stepper")    ; for function = Into
;(bind-key "Stepper Step To Call" "Meta-Shift-F8":mode "Stepper")    ; don't step fn's ar
;(bind-key "Stepper Step To Value"    "Shift-F8"	:mode "Stepper")
;(bind-key "Stepper Next"	      "Ctrl-F9"	:mode "Stepper")
;(bind-key "Stepper Step To End" "Ctrl-Shift-F8" :mode "Stepper") ; Step out
;(bind-key "Stepper Step To Cursor"    "Ctrl-F8"	:mode "Stepper")
;(bind-key "Stepper Continue"		"F5"	:mode "Stepper")
;(bind-key "Stepper Breakpoint"		"F9"	:mode "Stepper")
;Stepper Show Current Source
;Stepper Macroexpand
;Stepper Undo Macroexpand
;Stepper Restart

(let ((lw:*handle-warn-on-redefinition* :quiet))
 (defmethod capi:interface-display :before ((self lispworks-tools::stepper))
 ;;; Stepper menu
  (when-let (component (capi::find-menu-item self '((:menu "Stepper") 0)))
    (setf
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 0)) #\F8
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 1)) #\Meta-F8
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 2)) #\Meta-Shift-F8
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 3)) #\Shift-F8
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 4)) #\Ctrl-F9
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 5)) #\Ctrl-Shift-F8
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 6)) #\Ctrl-F8
     (CAPI::MENU-ITEM-ACCELERATOR (capi::find-menu-item component 7)) #\F5)
    (when-let (item (capi::find-menu-item self '((:menu "Stepper") 1 0 0)))
      (setf (CAPI::MENU-ITEM-ACCELERATOR item) #\F9)) ))
)

;;; I think you'd need to add a command to do this. Here's an attempt. It
;;; operates on the region since that's how Comment Region works and
;;; deletes continuous comment prefixes of any length.
;;; Alternatively you might want to add commands which comment and
;;; uncomment a line. 

(defcommand "Delete Comment Prefix in Region" (p)
     "Delete all sequences of ; at start of each line in region"
     "Delete comment prefixes"
  (declare (ignore p))
  (let ((buffer (editor:current-buffer)))
    (editor:with-point 
       ((start (editor:buffer-point buffer))
        (end (editor:buffer-mark buffer)))
      (when (editor:point> start end)
        (rotatef start end)) ; so point<= start end
      (editor:collect-undo buffer
        (loop while (editor:point< start end)
              do
              (unless (editor:start-line-p start)               ; ensure at start of line
                (editor:line-start start))
              (editor:with-point ((end1 start))
                ;; move a point to end of command prefix
                (loop while (eq (editor:character-at end1 0) #\;)
                      do (editor:character-offset end1 1))
                (unless (point= end1 start)
                  (when (eq (editor:character-at end1 0) #\Space); remove separating space
                    (editor:character-offset end1 1)))         ; if any
                (editor:delete-between-points start end1))     ; delete the comment prefix
           
              (editor:line-offset start 1))))))                ; move to next line

(bind-key "Comment Region" #\Ctrl-\;)
;; The number of ";" inserted is controlled by the prefix argument. Soo what you want:
;;    Control-U 3 Control-;

(bind-key "Delete Comment Prefix in Region" #\Ctrl-\:)

(defcommand "Insert Space and Show Arglist" (p)
     "Display the argument list in the echo area a while after 
 inserting Space to the right of the function."
     "Display the argument list."
   (editor:self-insert-command p #\Space)
   (editor:with-point ((temp1 (editor:current-point))
                       (temp2 (editor:current-point)))
     (when (editor:form-offset temp1 -1)
       (editor:character-offset temp1 -1)
       (let ((str (editor:points-to-string temp1 temp2)))
         (when (eq #\( (schar str 0))
           (when-let (symbol (or (find-symbol (nstring-upcase (subseq str 1
                                                                      (1- (length str))))
                                              (editor::buffer-package-to-use temp1))
                                 (ignore-errors (read-from-string str nil nil :start 1))))
             (when (and (symbolp symbol) (fboundp symbol))
               (editor:function-arglist-command nil symbol))))))))
               ;(editor:message ""))))))))

(bind-key "Insert Space and Show Arglist" #\Space :mode "Lisp")
(bind-key "Insert Space and Show Arglist" #\Space :mode "Execute")

#||
(defcommand "Web Help on Symbol" (p)
     "Acts like menu Help > On Symbol" ""
  (declare (ignore p))
  (let* ((menu (find :help-menu (capi:interface-menu-bar-items capi::*top-level-interface*)
                     :key #'capi:capi-object-name)))
    (when menu                                       ; menu bar is visible
      (let* ((component (find :web-help-component (capi:menu-items menu)
                              :key #'capi:capi-object-name))
             (item (capi:get-collection-item component 0)))
        (when (capi:menu-object-enabled item)
          (funcall (capi:callbacks-selection-callback item) capi::*top-level-interface*))))))
    
(bind-key "Web Help on Symbol" "ctrl-F1")
||#

